@@ -12,6 +12,7 @@ _("Citizen Service Number"),
 _("Code Insee"),
 _("CPR-nummer"),
 _("Czech Republic"),
+_("Demoworld"),
 _("Denmark"),
 _("Europe"),
 _("Fiscal code"),
@@ -34,5 +35,6 @@ _("Square number"),
 _("Switzerland"),
 _("Tax number"),
 _("Taxpayer identification number"),
+_("Testland"),
 _("United States of America (USA)"),
 };
