@@ -36,7 +36,7 @@
  * @param auth authentication data to use
  * @return #GNUNET_OK in success
  */
-int
+enum GNUNET_GenericReturnValue
 ANASTASIS_EUFIN_setup_auth_ (
   CURL *easy,
   const struct ANASTASIS_EUFIN_AuthenticationData *auth);
