@@ -126,16 +126,6 @@ fi
 
 echo "OK"
 
-echo -n "Test user attributes collection in a recovery state ..."
-anastasis-reducer -a \
-  '{"identity_attributes": {
-    "full_name": "Max Musterman",
-    "sq_number": "4",
-    "birthdate": "2000-01-01"}}' \
-  enter_user_attributes resources/02-recovery.json $TFILE 2> /dev/null && exit_fail "Expected recovery to fail due to lacking policy data"
-
-echo "OK"
-
 rm -f $TFILE
 
 exit 0
