@@ -15,6 +15,27 @@ function exit_fail() {
     exit 1
 }
 
+# Stop libeufin sandbox and nexus (if running)
+function stop_libeufin()
+{
+    echo "Stopping libeufin..."
+    if test -f libeufin-sandbox.pid
+    then
+        PID=`cat libeufin-sandbox.pid 2> /dev/null`
+        echo "Killing libeufin sandbox $PID"
+        rm libeufin-sandbox.pid
+        kill $PID 2> /dev/null || true
+    fi
+    if test -f libeufin-nexus.pid
+    then
+        PID=`cat libeufin-nexus.pid 2> /dev/null`
+        echo "Killing libeufin nexus $PID"
+        rm libeufin-nexus.pid
+        kill $PID 2> /dev/null || true
+    fi
+    echo "Stopping libeufin DONE"
+}
+
 # Cleanup to run whenever we exit
 function cleanup()
 {
@@ -22,6 +43,7 @@ function cleanup()
     do
         kill $n 2> /dev/null || true
     done
+    stop_libeufin
     rm -rf $CONF $WALLET_DB $R1FILE $R2FILE $B1FILE $B2FILE $TMP_DIR
     wait
 }
@@ -91,8 +113,8 @@ taler-exchange-httpd -h > /dev/null || exit_skip " taler-exchange required"
 taler-merchant-httpd -h > /dev/null || exit_skip " taler-merchant required"
 echo " FOUND"
 
-echo -n "Testing for taler-bank-manage"
-taler-bank-manage --help >/dev/null </dev/null || exit_skip " MISSING"
+echo -n "Testing for taler-bank-manage-testing"
+taler-bank-manage-testing --help >/dev/null </dev/null || exit_skip " MISSING"
 echo " FOUND"
 echo -n "Testing for taler-wallet-cli"
 taler-wallet-cli -v >/dev/null </dev/null || exit_skip " MISSING"
@@ -140,8 +162,8 @@ MASTER_PUB=`gnunet-ecc -p $MASTER_PRIV_FILE`
 EXCHANGE_URL=`taler-config -c $CONF -s EXCHANGE -o BASE_URL`
 MERCHANT_PORT=`taler-config -c $CONF -s MERCHANT -o PORT`
 MERCHANT_URL=http://localhost:${MERCHANT_PORT}/
-BANK_PORT=`taler-config -c $CONF -s BANK -o HTTP_PORT`
-BANK_URL=http://localhost:${BANK_PORT}/
+NEXUS_PORT=8082
+BANK_URL=http://localhost:1${NEXUS_PORT}/
 AUDITOR_URL=http://localhost:8083/
 AUDITOR_PRIV_FILE=`taler-config -f -c $CONF -s AUDITOR -o AUDITOR_PRIV_FILE`
 AUDITOR_PRIV_DIR=`dirname $AUDITOR_PRIV_FILE`
@@ -176,13 +198,14 @@ echo " OK"
 
 # Launch services
 echo -n "Launching taler services ..."
-taler-bank-manage-testing $CONF postgres:///$TALER_DB serve > taler-bank.log 2> taler-bank.err &
+rm -f $TALER_DB
+taler-bank-manage-testing $NEXUS_PORT $TALER_DB $EXCHANGE_URL $CONF > taler-bank.log 2> taler-bank.err &
 taler-exchange-secmod-eddsa -c $CONF 2> taler-exchange-secmod-eddsa.log &
 taler-exchange-secmod-rsa -c $CONF 2> taler-exchange-secmod-rsa.log &
 taler-exchange-secmod-cs -c $CONF 2> taler-exchange-secmod-cs.log &
-taler-exchange-httpd -c $CONF 2> taler-exchange-httpd.log &
+taler-exchange-httpd -L INFO -c $CONF 2> taler-exchange-httpd.log &
 taler-merchant-httpd -c $CONF -L INFO 2> taler-merchant-httpd.log &
-taler-exchange-wirewatch -c $CONF 2> taler-exchange-wirewatch.log &
+taler-exchange-wirewatch --longpoll-timeout=1s -L INFO -c $CONF 2> taler-exchange-wirewatch.log &
 taler-auditor-httpd -L INFO -c $CONF 2> taler-auditor-httpd.log &
 
 echo " OK"
@@ -201,7 +224,7 @@ do
     sleep 0.2
     OK=0
     # bank
-    wget --tries=1 --timeout=1 http://localhost:8082/ -o /dev/null -O /dev/null >/dev/null || continue
+    wget --tries=1 --timeout=1 http://localhost:18082/ -o /dev/null -O /dev/null >/dev/null || continue
     OK=1
     break
 done
@@ -238,9 +261,9 @@ echo -n "Setting up keys ..."
 taler-exchange-offline -c $CONF \
   download \
   sign \
-  enable-account payto://x-taler-bank/localhost/Exchange \
+  enable-account `taler-config -c $CONF -s exchange-account-1 -o PAYTO_URI` \
   enable-auditor $AUDITOR_PUB $AUDITOR_URL "TESTKUDOS Auditor" \
-  wire-fee now x-taler-bank TESTKUDOS:0.01 TESTKUDOS:0.01 TESTKUDOS:0.01 \
+  wire-fee now iban TESTKUDOS:0.01 TESTKUDOS:0.01 \
   upload &> taler-exchange-offline.log
 
 echo -n "."
@@ -295,7 +318,7 @@ echo "OK"
 echo -n "Configuring merchant instance ..."
 # Setup merchant
 
-curl -H "Content-Type: application/json" -X POST -d '{"auth":{"method":"external"},"payto_uris":["payto://x-taler-bank/localhost/43"],"id":"default","name":"default","address":{},"jurisdiction":{},"default_max_wire_fee":"TESTKUDOS:1", "default_max_deposit_fee":"TESTKUDOS:1","default_wire_fee_amortization":1,"default_wire_transfer_delay":{"d_ms" : 3600000},"default_pay_delay":{"d_ms": 3600000}}' http://localhost:9966/management/instances
+curl -H "Content-Type: application/json" -X POST -d '{"auth":{"method":"external"},"payto_uris":["payto://iban/SANDBOXX/DE648226?receiver-name=anastasis"],"id":"default","name":"default","address":{},"jurisdiction":{},"default_max_wire_fee":"TESTKUDOS:1", "default_max_deposit_fee":"TESTKUDOS:1","default_wire_fee_amortization":1,"default_wire_transfer_delay":{"d_us" : 3600000000},"default_pay_delay":{"d_us": 3600000000}}' http://localhost:9966/management/instances
 
 
 echo " DONE"
@@ -370,17 +393,18 @@ echo " OK"
 
 echo -n "Preparing wallet"
 rm $WALLET_DB
-taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB api 'withdrawTestBalance' \
+taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB api --expect-success 'withdrawTestBalance' \
   "$(jq -n '
     {
         amount: "TESTKUDOS:100",
         bankBaseUrl: $BANK_URL,
         exchangeBaseUrl: $EXCHANGE_URL
     }' \
-    --arg BANK_URL "$BANK_URL" \
+    --arg BANK_URL "${BANK_URL}demobanks/default/access-api/" \
     --arg EXCHANGE_URL "$EXCHANGE_URL"
-  )" 2> /dev/null >/dev/null
-taler-wallet-cli --wallet-db=$WALLET_DB run-until-done 2>/dev/null >/dev/null
+  )" 2> wallet-withdraw.err > wallet-withdraw.out
+taler-wallet-cli --wallet-db=$WALLET_DB \
+  run-until-done 2>wallet-withdraw-finish.err >wallet-withdraw-finish.out
 echo " OK"
 
 echo -en "Making payments for truth uploads ... "
@@ -390,12 +414,14 @@ do
     PAY_URI=`jq --argjson INDEX $INDEX -r -e '.payments[$INDEX]' < $B2FILE`
     # run wallet CLI
     echo -n "$INDEX"
-    taler-wallet-cli --wallet-db=$WALLET_DB handle-uri $PAY_URI -y 2>/dev/null >/dev/null
+    taler-wallet-cli --wallet-db=$WALLET_DB \
+       handle-uri $PAY_URI -y 2>wallet-pay-truth-$INDEX.err >wallet-pay-truth-$INDEX.out
     echo -n ", "
 done
 echo "OK"
-echo -e "Running wallet run-pending..."
-taler-wallet-cli --wallet-db=$WALLET_DB run-pending 2>/dev/null >/dev/null
+echo -e "Running wallet run-until-done..."
+taler-wallet-cli --wallet-db=$WALLET_DB \
+       run-until-done 2>wallet-pay-truth-finish-$INDEX.err >wallet-pay-truth-finish-$INDEX.out
 echo -e "Payments done"
 
 export B2FILE
@@ -413,12 +439,13 @@ do
     PAY_URI=`jq --argjson INDEX $INDEX -r -e '.policy_payment_requests[$INDEX].payto' < $B2FILE`
     # run wallet CLI
     echo -n "$INDEX"
-    taler-wallet-cli --wallet-db=$WALLET_DB handle-uri $PAY_URI -y 2>/dev/null >/dev/null
+    taler-wallet-cli --wallet-db=$WALLET_DB handle-uri $PAY_URI -y 2>wallet-pay-policy-$INDEX.err >wallet-pay-policy-$INDEX.out
     echo -n ", "
 done
 echo " OK"
-echo -en "Running wallet run-pending..."
-taler-wallet-cli --wallet-db=$WALLET_DB run-pending 2>/dev/null >/dev/null
+echo -en "Running wallet run-until-done..."
+taler-wallet-cli --wallet-db=$WALLET_DB \
+   run-until-done 2>wallet-pay-policy-finish.err >wallet-pay-policy-finish.out
 echo -e " payments DONE"
 
 echo -en "Try to upload again ..."
