@@ -1,16 +1,16 @@
 /*
   This file is part of Anastasis
-  Copyright (C) 2019-2021 Anastasis SARL
+  Copyright (C) 2019-2022 Anastasis SARL
 
   Anastasis is free software; you can redistribute it and/or modify it under the
-  terms of the GNU Affero General Public License as published by the Free Software
+  terms of the GNU General Public License as published by the Free Software
   Foundation; either version 3, or (at your option) any later version.
 
   Anastasis is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
+  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
-  You should have received a copy of the GNU Lesser General Public License along with
+  You should have received a copy of the GNU General Public License along with
   Anastasis; see the file COPYING.LIB.  If not, see <http://www.gnu.org/licenses/>
 */
 /**
@@ -61,11 +61,6 @@ struct ANASTASIS_Config
    */
   const char *business_name;
 
-  /**
-   * Currency used for payments by the server.
-   */
-  const char *currency;
-
   /**
    * Array of authorization methods supported by the server.
    */
@@ -98,9 +93,9 @@ struct ANASTASIS_Config
   struct TALER_Amount liability_limit;
 
   /**
-   * Server salt.
+   * Provider salt.
    */
-  struct ANASTASIS_CRYPTO_ProviderSaltP salt;
+  struct ANASTASIS_CRYPTO_ProviderSaltP provider_salt;
 
 };
 
@@ -155,6 +150,98 @@ ANASTASIS_config_cancel (struct ANASTASIS_ConfigOperation *co);
 /****** POLICY API ******/
 
 
+/**
+ * Detailed meta data result.
+ */
+struct ANASTASIS_MetaDataEntry
+{
+
+  /**
+   * Timestamp of the backup at the server.
+   */
+  struct GNUNET_TIME_Timestamp server_time;
+
+  /**
+   * The encrypted meta data we downloaded.
+   */
+  const void *meta_data;
+
+  /**
+   * Number of bytes in @e meta_data.
+   */
+  size_t meta_data_size;
+
+  /**
+   * Policy version this @e meta_data is for.
+   */
+  uint32_t version;
+};
+
+
+/**
+ * Detailed results for meta data download.
+ */
+struct ANASTASIS_MetaDownloadDetails
+{
+
+  /**
+   * Version-sorted array of meta data we downloaded.
+   */
+  const struct ANASTASIS_MetaDataEntry *metas;
+
+  /**
+   * Number of entries in @e metas.
+   */
+  size_t metas_length;
+
+};
+
+
+/**
+ * Callback to process a GET /policy/$POL/meta request
+ *
+ * @param cls closure
+ * @param http_status HTTP status code for this request
+ * @param dd the response details
+ */
+typedef void
+(*ANASTASIS_PolicyMetaLookupCallback) (
+  void *cls,
+  unsigned int http_status,
+  const struct ANASTASIS_MetaDownloadDetails *dd);
+
+
+/**
+ * Does a GET /policy/$POL/meta.
+ *
+ * @param ctx execution context
+ * @param backend_url base URL of the merchant backend
+ * @param anastasis_pub public key of the user's account
+ * @param max_version maximum version number to fetch
+ * @param cb callback which will work the response gotten from the backend
+ * @param cb_cls closure to pass to the callback
+ * @return handle for this operation, NULL upon errors
+ */
+struct ANASTASIS_PolicyMetaLookupOperation *
+ANASTASIS_policy_meta_lookup (
+  struct GNUNET_CURL_Context *ctx,
+  const char *backend_url,
+  const struct ANASTASIS_CRYPTO_AccountPublicKeyP *anastasis_pub,
+  uint32_t max_version,
+  ANASTASIS_PolicyMetaLookupCallback cb,
+  void *cb_cls);
+
+
+/**
+ * Cancel a GET /policy/$POL/meta request.
+ *
+ * @param plo cancel the policy lookup operation
+ */
+void
+ANASTASIS_policy_meta_lookup_cancel (
+  struct ANASTASIS_PolicyMetaLookupOperation *plo);
+
+
 /**
  * Detailed results from the successful download.
  */
@@ -198,8 +285,7 @@ struct ANASTASIS_PolicyLookupOperation;
  *
  * @param cls closure
  * @param http_status HTTP status code for this request
- * @param ec anastasis-specific error code
- * @param obj the response body
+ * @param dd the response details
  */
 typedef void
 (*ANASTASIS_PolicyLookupCallback) (void *cls,
@@ -337,7 +423,7 @@ struct ANASTASIS_UploadDetails
        * At what time is the provider set to forget this
        * policy (because the account expires)?
        */
-      struct GNUNET_TIME_Absolute policy_expiration;
+      struct GNUNET_TIME_Timestamp policy_expiration;
 
       /**
        * Version number of the resulting policy.
@@ -371,8 +457,7 @@ struct ANASTASIS_UploadDetails
  * Callback to process a POST /policy request
  *
  * @param cls closure
- * @param http_status HTTP status code for this request
- * @param obj the decoded response body
+ * @param up the decoded response body
  */
 typedef void
 (*ANASTASIS_PolicyStoreCallback) (void *cls,
@@ -387,6 +472,8 @@ typedef void
  * @param anastasis_priv private key of the user's account
  * @param recovery_data policy data to be stored
  * @param recovery_data_size number of bytes in @a recovery_data
+ * @param recovery_meta_data policy meta data to be stored
+ * @param recovery_meta_data_size number of bytes in @a recovery_meta_data
  * @param payment_years_requested for how many years would the client like the service to store the truth?
  * @param payment_secret payment identifier of last payment
  * @param payment_timeout how long to wait for the payment, use
@@ -402,6 +489,8 @@ ANASTASIS_policy_store (
   const struct ANASTASIS_CRYPTO_AccountPrivateKeyP *anastasis_priv,
   const void *recovery_data,
   size_t recovery_data_size,
+  const void *recovery_meta_data,
+  size_t recovery_meta_data_size,
   uint32_t payment_years_requested,
   const struct ANASTASIS_PaymentSecretP *payment_secret,
   struct GNUNET_TIME_Relative payment_timeout,
@@ -422,241 +511,6 @@ ANASTASIS_policy_store_cancel (
 /****** TRUTH API ******/
 
 
-/**
- * Operational status.
- */
-enum ANASTASIS_KeyShareDownloadStatus
-{
-  /**
-   * We got the encrypted key share.
-   */
-  ANASTASIS_KSD_SUCCESS = 0,
-
-  /**
-   * Payment is needed to proceed with the recovery.
-   */
-  ANASTASIS_KSD_PAYMENT_REQUIRED,
-
-  /**
-   * The provided answer was wrong or missing. Instructions for
-   * getting a good answer may be provided.
-   */
-  ANASTASIS_KSD_INVALID_ANSWER,
-
-  /**
-   * To answer the challenge, the client should be redirected to
-   * the given URL.
-   */
-  ANASTASIS_KSD_REDIRECT_FOR_AUTHENTICATION,
-
-  /**
-   * The provider had an error.
-   */
-  ANASTASIS_KSD_SERVER_ERROR,
-
-  /**
-   * The provider claims we made an error.
-   */
-  ANASTASIS_KSD_CLIENT_FAILURE,
-
-  /**
-   * The provider does not know this truth.
-   */
-  ANASTASIS_KSD_TRUTH_UNKNOWN,
-
-  /**
-   * Too many attempts to solve the challenge were made in a short
-   * time. Try again later.
-   */
-  ANASTASIS_KSD_RATE_LIMIT_EXCEEDED,
-
-  /**
-   * The user did not satisfy the (external)
-   * authentication check until the request timeout
-   * was reached. The client should try again later.
-   */
-  ANASTASIS_KSD_AUTHENTICATION_TIMEOUT,
-
-  /**
-   * The plugin provided external challenge instructions
-   * that should be followed. They are method-specific.
-   */
-  ANASTASIS_KSD_EXTERNAL_CHALLENGE_INSTRUCTIONS
-
-};
-
-
-/**
- * Detailed results from the successful download.
- */
-struct ANASTASIS_KeyShareDownloadDetails
-{
-
-  /**
-   * Operational status.
-   */
-  enum ANASTASIS_KeyShareDownloadStatus status;
-
-  /**
-   * Anastasis URL that returned the @e status.
-   */
-  const char *server_url;
-
-  /**
-   * Details depending on @e status.
-   */
-  union
-  {
-
-    /**
-     * The encrypted key share (if @e status is #ANASTASIS_KSD_SUCCESS).
-     */
-    struct ANASTASIS_CRYPTO_EncryptedKeyShareP eks;
-
-    /**
-     * Response if the challenge still needs to be answered, and the
-     * instructions are provided inline (no redirection).
-     */
-    struct
-    {
-
-      /**
-       * HTTP status returned by the server.  #MHD_HTTP_ALREADY_REPORTED
-       * if the server did already send the challenge to the user,
-       * #MHD_HTTP_FORBIDDEN if the answer was wrong (or missing).
-       */
-      unsigned int http_status;
-
-      /**
-       * Response with server-side reply containing instructions for the user
-       */
-      const char *body;
-
-      /**
-       * Content-type: mime type of @e body, NULL if server did not provide any.
-       */
-      const char *content_type;
-
-      /**
-       * Number of bytes in @e body.
-       */
-      size_t body_size;
-
-    } open_challenge;
-
-    /**
-     * URL with instructions for the user to satisfy the challenge, if
-     * @e status is #ANASTASIS_KSD_REDIRECT_FOR_AUTHENTICATION.
-     */
-    const char *redirect_url;
-
-    /**
-     * Response with instructions for how to pay, if
-     * @e status is #ANASTASIS_KSD_PAYMENT_REQUIRED.
-     */
-    struct
-    {
-
-      /**
-       * "taler://pay" URL with details how to pay for the challenge.
-       */
-      const char *taler_pay_uri;
-
-      /**
-       * The order ID from @e taler_pay_uri.
-       */
-      struct ANASTASIS_PaymentSecretP payment_secret;
-
-    } payment_required;
-
-
-    /**
-     * Response with details about a server-side failure, if
-     * @e status is #ANASTASIS_KSD_SERVER_ERROR,
-     * #ANASTASIS_KSD_CLIENT_FAILURE or #ANASTASIS_KSD_TRUTH_UNKNOWN.
-     */
-    struct
-    {
-
-      /**
-       * HTTP status returned by the server.
-       */
-      unsigned int http_status;
-
-      /**
-       * Taler-specific error code.
-       */
-      enum TALER_ErrorCode ec;
-
-    } server_failure;
-
-    /**
-     * External challenge instructions, if @e status is
-     * #ANASTASIS_KSD_EXTERNAL_CHALLENGE_INSTRUCTIONS.
-     */
-    const json_t *external_challenge;
-
-  } details;
-};
-
-
-/**
- * Handle for a GET /truth operation.
- */
-struct ANASTASIS_KeyShareLookupOperation;
-
-
-/**
- * Callback to process a GET /truth request
- *
- * @param cls closure
- * @param http_status HTTP status code for this request
- * @param kdd details about the key share
- */
-typedef void
-(*ANASTASIS_KeyShareLookupCallback) (
-  void *cls,
-  const struct ANASTASIS_KeyShareDownloadDetails *kdd);
-
-
-/**
- * Does a GET /truth.
- *
- * @param ctx execution context
- * @param backend_url base URL of the merchant backend
- * @param truth_uuid identification of the Truth
- * @param truth_key Key used to Decrypt the Truth on the Server
- * @param payment_secret secret from the previously done payment NULL to trigger payment
- * @param timeout how long to wait for the payment, use
- *           #GNUNET_TIME_UNIT_ZERO to let the server pick
- * @param hashed_answer hashed answer to the challenge
- * @param cb callback which will work the response gotten from the backend
- * @param cb_cls closure to pass to the callback
- * @return handle for this operation, NULL upon errors
- */
-struct ANASTASIS_KeyShareLookupOperation *
-ANASTASIS_keyshare_lookup (
-  struct GNUNET_CURL_Context *ctx,
-  const char *backend_url,
-  const struct ANASTASIS_CRYPTO_TruthUUIDP *truth_uuid,
-  const struct ANASTASIS_CRYPTO_TruthKeyP *truth_key,
-  const struct ANASTASIS_PaymentSecretP *payment_secret,
-  struct GNUNET_TIME_Relative timeout,
-  const struct GNUNET_HashCode *hashed_answer,
-  ANASTASIS_KeyShareLookupCallback cb,
-  void *cb_cls);
-
-
-/**
- * Cancel a GET /truth request.
- *
- * @param kslo cancel the key share lookup operation
- */
-void
-ANASTASIS_keyshare_lookup_cancel (
-  struct ANASTASIS_KeyShareLookupOperation *kslo);
-
-
 /**
  * Handle for a POST /truth operation.
  */
@@ -718,4 +572,358 @@ ANASTASIS_truth_store_cancel (
   struct ANASTASIS_TruthStoreOperation *tso);
 
 
+/**
+ * Possible ways how to proceed with a challenge.
+ */
+enum ANASTASIS_ChallengeDetailType
+{
+
+  /**
+   * A challenge TAN was written to a file.
+   * The name of the file is provided.
+   */
+  ANASTASIS_CS_FILE_WRITTEN,
+
+  /**
+   * A challenge TAN was sent to the customer.
+   * A hint may be provided as to the address used.
+   */
+  ANASTASIS_CS_TAN_SENT,
+
+  /**
+   * A challenge TAN was already recently sent to the customer.
+   * A hint may be provided as to the address used.
+   */
+  ANASTASIS_CS_TAN_ALREADY_SENT,
+
+  /**
+   * The customer should wire funds to the bank
+   * account address provided.
+   */
+  ANASTASIS_CS_WIRE_FUNDS
+
+};
+
+
+/**
+ * This structure contains information about where to wire the funds
+ * to authenticate as well as a hint as to which bank account to send
+ * the funds from.
+ */
+struct ANASTASIS_WireFundsDetails
+{
+
+  /**
+   * Answer code expected.
+   */
+  uint64_t answer_code;
+
+  /**
+   * How much should be sent.
+   */
+  struct TALER_Amount amount;
+
+  /**
+   * IBAN where to send the funds.
+   */
+  const char *target_iban;
+
+  /**
+   * Name of the business receiving the funds.
+   */
+  const char *target_business_name;
+
+  /**
+   * Wire transfer subject to use.
+   */
+  const char *wire_transfer_subject;
+
+};
+
+
+/**
+ * Information returned for a POST /truth/$TID/challenge request.
+ */
+struct ANASTASIS_TruthChallengeDetails
+{
+  /**
+   * HTTP status returned by the server.
+   */
+  unsigned int http_status;
+
+  /**
+   * Taler-specific error code, #TALER_EC_NONE on success.
+   */
+  enum TALER_ErrorCode ec;
+
+  /**
+   * Full response in JSON, if provided.
+   */
+  const json_t *response;
+
+  /**
+   * Details depending on @e http_status.
+   */
+  union
+  {
+
+    /**
+     * Information for @e http_status of #MHD_HTTP_OK.
+     */
+    struct
+    {
+      /**
+       * Meta-state about how the challenge was
+       * initiated and what is to be done next.
+       */
+      enum ANASTASIS_ChallengeDetailType cs;
+
+      /**
+       * Details depending on @e cs.
+       */
+      union
+      {
+
+        /**
+         * If @e cs is #ANASTASIS_CS_FILE_WRITTEN, this
+         * is the filename with the challenge code.
+         */
+        const char *challenge_filename;
+
+        /**
+         * If @e cs is #ANASTASIS_CS_TAN_SENT, this
+         * is human-readable information as to where
+         * the TAN was sent.
+         */
+        const char *tan_address_hint;
+
+        /**
+         * If @e cs is #ANASTASIS_CS_WIRE_FUNDS, this
+         * structure contains information about where
+         * to wire the funds to authenticate as well
+         * as a hint as to which bank account to send
+         * the funds from.
+         */
+        struct ANASTASIS_WireFundsDetails wire_funds;
+
+      } details;
+
+    } success;
+
+    /**
+     * Information returne if @e http_status is #MHD_HTTP_PAYMENT_REQUIRED
+     */
+    struct
+    {
+      /**
+       * A taler://pay/-URI with a request to pay the annual fee for
+       * the service.  Returned if @e us is #ANASTASIS_US_PAYMENT_REQUIRED.
+       */
+      const char *payment_request;
+
+      /**
+       * The payment secret (aka order ID) extracted from the @e payment_request.
+       */
+      struct ANASTASIS_PaymentSecretP ps;
+
+      /**
+       * Data extracted from the payto:// URI.
+       */
+      const struct TALER_MERCHANT_PayUriData *pd;
+
+    } payment_required;
+
+  } details;
+
+};
+
+
+/**
+ * Handle for a POST /truth/$TID/challenge operation.
+ */
+struct ANASTASIS_TruthChallengeOperation;
+
+
+/**
+ * Callback to process a POST /truth/$TID/challenge response.
+ *
+ * @param cls closure
+ * @param tcd details about the key share
+ */
+typedef void
+(*ANASTASIS_TruthChallengeCallback) (
+  void *cls,
+  const struct ANASTASIS_TruthChallengeDetails *tcd);
+
+
+/**
+ * Makes a POST /truth/$TID/challenge request.
+ *
+ * @param ctx execution context
+ * @param backend_url base URL of the merchant backend
+ * @param truth_uuid identification of the Truth
+ * @param truth_key Key used to Decrypt the Truth on the Server
+ * @param payment_secret secret from the previously done payment NULL to trigger payment
+ * @param cb callback which will work the response gotten from the backend
+ * @param cb_cls closure to pass to the callback
+ * @return handle for this operation, NULL upon errors
+ */
+struct ANASTASIS_TruthChallengeOperation *
+ANASTASIS_truth_challenge (
+  struct GNUNET_CURL_Context *ctx,
+  const char *backend_url,
+  const struct ANASTASIS_CRYPTO_TruthUUIDP *truth_uuid,
+  const struct ANASTASIS_CRYPTO_TruthKeyP *truth_key,
+  const struct ANASTASIS_PaymentSecretP *payment_secret,
+  ANASTASIS_TruthChallengeCallback cb,
+  void *cb_cls);
+
+
+/**
+ * Cancel a POST /truth/$TID/challenge request.
+ *
+ * @param[in] tco operation to cancel
+ */
+void
+ANASTASIS_truth_challenge_cancel (
+  struct ANASTASIS_TruthChallengeOperation *tco);
+
+
+/**
+ * Information returned for a POST /truth/$TID/solve request.
+ */
+struct ANASTASIS_TruthSolveReply
+{
+
+  /**
+   * HTTP status returned by the server.
+   */
+  unsigned int http_status;
+
+  /**
+   * Taler-specific error code, #TALER_EC_NONE on success.
+   */
+  enum TALER_ErrorCode ec;
+
+  /**
+   * Details depending on @e http_status.
+   */
+  union
+  {
+
+    /**
+     * Information returned if @e http_status is #MHD_HTTP_OK.
+     */
+    struct
+    {
+
+      /**
+       * The encrypted key share.
+       */
+      struct ANASTASIS_CRYPTO_EncryptedKeyShareP eks;
+
+    } success;
+
+    /**
+     * Information returne if @e http_status is #MHD_HTTP_PAYMENT_REQUIRED
+     */
+    struct
+    {
+      /**
+       * A taler://pay/-URI with a request to pay the annual fee for
+       * the service.  Returned if @e us is #ANASTASIS_US_PAYMENT_REQUIRED.
+       */
+      const char *payment_request;
+
+      /**
+       * The payment secret (aka order ID) extracted from the @e payment_request.
+       */
+      struct ANASTASIS_PaymentSecretP ps;
+
+      /**
+       * Data extracted from the payto:// URI.
+       */
+      const struct TALER_MERCHANT_PayUriData *pd;
+
+    } payment_required;
+
+    /**
+     * Information returne if @e http_status is #MHD_HTTP_TOO_MANY_REQUESTS.
+     */
+    struct
+    {
+
+      /**
+       * How many requests are allowed at most per @e request_frequency?
+       */
+      uint32_t request_limit;
+
+      /**
+       * Frequency at which requests are allowed / new challenges are
+       * created.
+       */
+      struct GNUNET_TIME_Relative request_frequency;
+    } too_many_requests;
+
+  } details;
+
+};
+
+
+/**
+ * Handle for a POST /truth/$TID/solve operation.
+ */
+struct ANASTASIS_TruthSolveOperation;
+
+
+/**
+ * Callback to process a POST /truth/$TID/solve response.
+ *
+ * @param cls closure
+ * @param kdd details about the key share
+ */
+typedef void
+(*ANASTASIS_TruthSolveCallback) (
+  void *cls,
+  const struct ANASTASIS_TruthSolveReply *trs);
+
+
+/**
+ * Makes a POST /truth/$TID/solve request.
+ *
+ * @param ctx execution context
+ * @param backend_url base URL of the merchant backend
+ * @param truth_uuid identification of the Truth
+ * @param truth_key Key used to Decrypt the Truth on the Server
+ * @param payment_secret secret from the previously done payment NULL to trigger payment
+ * @param timeout how long to wait for the payment, use
+ *           #GNUNET_TIME_UNIT_ZERO to let the server pick
+ * @param hashed_answer hashed answer to the challenge
+ * @param cb callback which will work the response gotten from the backend
+ * @param cb_cls closure to pass to the callback
+ * @return handle for this operation, NULL upon errors
+ */
+struct ANASTASIS_TruthSolveOperation *
+ANASTASIS_truth_solve (
+  struct GNUNET_CURL_Context *ctx,
+  const char *backend_url,
+  const struct ANASTASIS_CRYPTO_TruthUUIDP *truth_uuid,
+  const struct ANASTASIS_CRYPTO_TruthKeyP *truth_key,
+  const struct ANASTASIS_PaymentSecretP *payment_secret,
+  struct GNUNET_TIME_Relative timeout,
+  const struct GNUNET_HashCode *hashed_answer,
+  ANASTASIS_TruthSolveCallback cb,
+  void *cb_cls);
+
+
+/**
+ * Cancel a POST /truth/$TID/solve request.
+ *
+ * @param[in] tso handle of the operation to cancel
+ */
+void
+ANASTASIS_truth_solve_cancel (
+  struct ANASTASIS_TruthSolveOperation *tso);
+
+
 #endif  /* _ANASTASIS_SERVICE_H */
