@@ -879,6 +879,9 @@ begin_provider_config_check (const char *cc,
                          cc)) )
       {
         /* skip */
+        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                    "Skipping provider restricted to country `%s'\n",
+                    restricted);
         continue;
       }
       if ( (NULL == restricted) &&
